#include <stdio.h>
#include <string>
#define _USE_MATH_DEFINES
#include <cmath>
#include <opencv2/opencv.hpp>
#include <opencv2/ccalib/omnidir.hpp>

int
main(int argc, char** argv)
{
	double max_theta = 235.0 / 2.0; // Kodak SP360 : View angle is 235 deg

	std::string inf, outf;

	if (argc <= 1)
	{
		std::cout << "Usage: omni2pers.exe [option] imagefile" << std::endl;
		std::cout << "     -o filename" << std::endl;
		std::cout << "     --help   display this help and exit" << std::endl;
		return (0);
	}

	int n = 0;
	while(++n<argc){
		switch (argv[n][0]) {
		case '-':
			if (argv[n][1] == 'o') outf = argv[++n];
			break;
		default:
			inf = argv[n];
		}
	}

	cv::Mat image = cv::imread(inf.c_str(), cv::IMREAD_COLOR);

	double iH = image.rows / 2;
	/*
	double r = iH / (1.0 + sin((max_theta - 90.0) * M_PI / 180.0));
	double f = iH - r;
	double iW = (int)(2.0 * M_PI * r * sin((max_theta - 90.0) * M_PI / 180.0));
	*/
	double r = iH / (max_theta * M_PI / 180.0);
	double f = r;
	double iW = (int)(2.0 * M_PI * f);

	int W = (int)iW;
	int H = (int)iH;

	cv::Mat K = (cv::Mat_<double>(3, 3) << -f, 0, image.rows/2, 0, -f, image.cols/2, 0, 0, 1); // 3x3 intrinsic parameters
	cv::Mat D = (cv::Mat_<double>(4, 1) << 0, 0, 0, 0); // 4x1 or similar distortion parameters
	cv::Mat xi = (cv::Mat_<double>(1, 1) << 0);

	xi = 1.0;
	H += H * sin((max_theta-90.0) * M_PI / 180.0);

	cv::Mat cyimage, map1, map2;
	//cv::InputArray R = cv::Matx33d::eye();
	//cv::InputArray P = cv::getOptimalNewCameraMatrix(K, D, image.size(), 1, cv::Size(W, H), 0);

	//cv::omnidir::initUndistortRectifyMap(K, D, xi, R, P, cv::Size(W, H), CV_16SC2, map1, map2, cv::omnidir::RECTIFY_CYLINDRICAL);
	cv::omnidir::initUndistortRectifyMap(K, D, xi, cv::Mat(), cv::Mat(), cv::Size(W, H), CV_16SC2, map1, map2, cv::omnidir::RECTIFY_CYLINDRICAL);

	cv::remap(image, cyimage, map1, map2, cv::INTER_LINEAR);

	if (!outf.empty())
		cv::imwrite(outf.c_str(), cyimage);
	else {
		cv::imshow("Image View", cyimage);
		cv::waitKey(0);
	}

	return 0;
}
